/* configEncoder.h */
#ifndef CONFIG_GHS_ENCODER_H__
#define CONFIG_GHS_ENCODER_H__
#include "GhsControlStructs.h"


// 


void cleanUpSystemInfo(s_SystemInfo** systemInfoPtr);

void cleanUpSystemInfoData(s_SystemInfoData** systemInfoDataPtr);

void cleanUpTimeInfo(s_TimeInfo** timeInfoPtr);

/**
 * Call this at the end of the program. Frees up all the allocated resources use in creating the TimeInfo Data
 * @param timeInfoDataPtr pointer a pointer of the application's s_TimeInfoData struct.
 */
void cleanUpTimeInfoData(s_TimeInfoData** timeInfoDataPtr);

/**
 * Sets up the time info for the device. Method allocates memory for the s_TimeInfo. If this method is called
 * twice the existing timeInfo is freed and a new one is created.
 * Only one time clock type can exist. It is also possible to support no time clock at all such as for 
 * continuous live measurements.
 * If more than one clock type is indicated the first one checked that is true takes precident. Order of checking is
 * usesAbsTime, then usesBoTime, then usesRelativeTime.
 * @param timeInfoPtr pointer to an s_TimeInfo *struct
 * @param usesAbsTime true if PHD clock uses absolute time
 * @param usesBoTime true if PHD clock uses base-offset time
 * @param usesRelativeTime true if PHD clock uses a relative time
 * @param allowSetTime true if PHD clock can be set. Applies only to absolute and base offset times. Ignored otherwise
 * @param timeResolution resolution between time ticks in micorseconds. E.g 1000 is 1/100th of a second. A value of 0 means unknown
 * @returns the populated TimeInfo struct.
 */
bool createTimeInfo(s_TimeInfo **timeInfoPtr, bool usesAbsTime, bool usesBoTime, bool usesRelativeTime, bool allowSetTime, unsigned long timeResolution);

bool createCurrentTimeDataBuffer(s_TimeInfoData** timeInfoDataPtr, s_TimeInfo *timeInfo);


bool createSystemInfo(s_SystemInfo **systemInfoPtr, unsigned short numberOfSpecializations);

bool setSystemIdentifierByte(s_SystemInfo **systemInfoPtr, unsigned char *systemId);

bool addSpecialization(s_SystemInfo **systemInfoPtr, unsigned short specializationTermCode, unsigned short version);

bool setRequiredSystemInfoStrings(s_SystemInfo **systemInfoPtr, char *manufacturerName, char *modelNumber);

bool setOptionalSystemInfoStrings(s_SystemInfo **systemInfoPtr, char *serialNumber, char *firmware, char *hardware, char *software);

/**
 *  The set value 'isRegulated' is only used if 'hasRegulationStatus' is true.
 */
bool setRegulationStatus(s_SystemInfo **systemInfoPtr, bool hasRegulationStatus, bool isRegulated);

/**
 * 16-digit HEX string. This method will put the system id in little endian order as bytes.
 */
bool setSystemIdentifier(s_SystemInfo **systemInfoPtr, char *systemId);

#if (USES_SYSTEM_AVAS == 1)
bool initializeSystemInfoAvas(s_SystemInfo** systemInfoPtr, unsigned short numberOfAvas);

bool addSystemInfoAvas(s_SystemInfo** systemInfoPtr, s_Avas *ava);
#endif

bool createSystemInfoData(s_SystemInfoData** systemInfoDataPtr, s_SystemInfo *systemInfo);


void cleanUpGhsMsmt(s_GhsMsmt** ghsMsmtPtr);

void cleanUpMsmtGroup(s_MsmtGroup** msmtGroupPtr);

void cleanUpMsmtGroupData(s_MsmtGroupData** msmtGroupDataPtr);

bool createMsmtGroup(s_MsmtGroup **msmtGroupPtr, bool hasTimeStamp, unsigned char numberOfGhsMsmts, unsigned char groupId);

bool setHeaderOptions(s_MsmtGroup **msmtGroupPtr, bool areSettings, bool hasPersonId, unsigned short personId);

bool setHeaderSupplementalTypes(s_MsmtGroup **msmtGroupPtr, unsigned long *supplementalTypes, unsigned short numberOfSupplementalTypes );

bool setHeaderSrcHandleRefs(s_MsmtGroup **msmtGroupPtr, unsigned short *srcHandleRefs, unsigned short numberOfSrcHandleRefs );

bool setHeaderDuration(s_MsmtGroup **msmtGroupPtr, s_MderFloat *duration );

#if (USES_HEADER_AVAS == 1)
bool initializeHeaderAvas(s_MsmtGroup **msmtGroupPtr, unsigned short numberOfAvas);

bool addHeaderAva(s_MsmtGroup** msmtGroupPtr, s_Avas* avaIn);
#endif

short addGhsMsmtToGroup(s_GhsMsmt *ghsMsmt, s_MsmtGroup **msmtGroupPtr);

#if(USES_NUMERIC == 1)
bool createNumericMsmt(s_GhsMsmt **ghsMsmtPtr, unsigned long type, bool isSfloat, unsigned short units);

bool updateNumericMsmt(s_GhsMsmt **ghsMsmtPtr, s_MderFloat *value, unsigned short msmt_id);

bool updateDataNumeric(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, s_MderFloat* value, unsigned short msmt_id);
#endif
#if(USES_COMPOUND == 1)
bool createCompoundNumericMsmt(s_GhsMsmt** ghsMsmtPtr, unsigned long type, bool isSfloat, unsigned short units, unsigned short numberOfComponents, s_Compound *compounds);

bool updateCompoundMsmt(s_GhsMsmt **ghsMsmtPtr, s_MderFloat *value, unsigned short msmt_id);

bool updateDataCompound(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, s_MderFloat* value, unsigned short msmt_id);
#endif
#if(USES_CODED == 1)
bool createCodedMsmt(s_GhsMsmt** ghsMsmtPtr, unsigned long type);

bool updateCodedEnum(s_GhsMsmt **ghsMsmtPtr, unsigned long code, unsigned short msmt_id);

bool updateDataCoded(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, unsigned long code, unsigned short msmt_id);
#endif
#if (USES_BITS16 == 1)
bool createBit16EnumMsmt(s_GhsMsmt** ghsMsmtPtr, unsigned long type, unsigned short state, unsigned short support);

bool updateBits16Enum(s_GhsMsmt** ghsMsmtPtr, unsigned short bits16, unsigned short msmt_id);

bool updateDataBits16(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, unsigned short bits16, unsigned short msmt_id);
#endif

#if (USES_BITS32 == 1)
bool createBit32EnumMsmt(s_GhsMsmt** ghsMsmtPtr, unsigned long type, unsigned long state, unsigned long support);

bool updateBits32Enum(s_GhsMsmt** ghsMsmtPtr, unsigned long bits32, unsigned short msmt_id);

bool updateDataBits32(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, unsigned long bits32, unsigned short msmt_id);
#endif

#if (USES_RTSA == 1)
bool createRtsaMsmt(s_GhsMsmt** ghsMsmtPtr, unsigned long type, unsigned short units, s_MderFloat* period, s_MderFloat* scaleFactor, s_MderFloat* offset,
    unsigned short numberOfSamples, unsigned char sampleSize);

bool updateRtsa(s_GhsMsmt** ghsMsmtPtr, unsigned char* samples, unsigned short msmt_id);

bool updateDataRtsa(s_MsmtGroupData** msmtGroupDataPtr, unsigned short msmtIndex, unsigned char* samples, unsigned short dataLength, unsigned short msmt_id);
#endif

bool setGhsMsmtSupplementalTypes(s_GhsMsmt **ghsMsmtPtr, unsigned long *supplementalTypes, unsigned short numberOfSupplementalTypes );

bool setGhsMsmtSrcHandleRefs(s_GhsMsmt **ghsMsmtPtr, unsigned short *srcHandleRefs, unsigned short numberOfSrcHandleRefs );

bool setGhsMsmtDuration(s_GhsMsmt **ghsMsmt, s_MderFloat *duration );

#if (USES_MSMT_AVAS == 1)
bool initializeGhsMsmtAvas(s_GhsMsmt **ghsMsmt, unsigned short numberOfAvas);

bool addGhsMsmtAva(s_GhsMsmt **ghsMsmt, s_Avas *ava);
#endif

bool createMsmtGroupDataArray(s_MsmtGroupData** msmtGroupPtr, s_MsmtGroup *msmtGroup, unsigned short packetType);

#endif  //CONFIG_GHS_ENCODER_H__
